@@ -1,7 +1,8 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import datetime
 import os
+import unicodedata
 from itertools import groupby
 from io import BytesIO
 
@@ -11,7 +12,7 @@ from lxml import etree
 from sql import Literal
 
 from trytond.pool import PoolMeta, Pool
-from trytond.model import ModelSQL, ModelView, Workflow, fields
+from trytond.model import ModelSQL, ModelView, Workflow, fields, dualmethod
 from trytond.pyson import Eval, If
 from trytond.transaction import Transaction
 from trytond.tools import reduce_ids, grouped_slice
@@ -43,6 +44,7 @@ class Journal:
             (None, ''),
             ('pain.001.001.03', 'pain.001.001.03'),
             ('pain.001.001.05', 'pain.001.001.05'),
+            ('pain.001.003.03', 'pain.001.003.03'),
             ], 'Payable Flavor', states={
             'required': Eval('process_method') == 'sepa',
             'invisible': Eval('process_method') != 'sepa',
@@ -53,6 +55,7 @@ class Journal:
             (None, ''),
             ('pain.008.001.02', 'pain.008.001.02'),
             ('pain.008.001.04', 'pain.008.001.04'),
+            ('pain.008.003.02', 'pain.008.003.02'),
             ], 'Receivable Flavor', states={
             'required': Eval('process_method') == 'sepa',
             'invisible': Eval('process_method') != 'sepa',
@@ -127,6 +130,9 @@ class Group:
         cls._error_messages.update({
                 'no_mandate': 'No valid mandate for payment "%s"',
                 })
+        cls._buttons.update({
+                'generate_message': {},
+                })
 
     def get_sepa_template(self):
         if self.kind == 'payable':
@@ -137,11 +143,9 @@ class Group:
     def process_sepa(self):
         pool = Pool()
         Payment = pool.get('account.payment')
-        Message = pool.get('account.payment.sepa.message')
         if self.kind == 'receivable':
-            payments = [p for p in self.payments if not p.sepa_mandate]
-            mandates = Payment.get_sepa_mandates(payments)
-            for payment, mandate in zip(payments, mandates):
+            mandates = Payment.get_sepa_mandates(self.payments)
+            for payment, mandate in zip(self.payments, mandates):
                 if not mandate:
                     self.raise_user_error('no_mandate', payment.rec_name)
                 # Write one by one becasue mandate.sequence_type must be
@@ -150,15 +154,27 @@ class Group:
                         'sepa_mandate': mandate,
                         'sepa_mandate_sequence_type': mandate.sequence_type,
                         })
-        tmpl = self.get_sepa_template()
-        if not tmpl:
-            raise NotImplementedError
-        if not self.sepa_messages:
-            self.sepa_messages = ()
-        message = tmpl.generate(group=self,
-            datetime=datetime).filter(remove_comment).render()
-        message = Message(message=message, type='in', state='waiting')
-        self.sepa_messages += (message,)
+        self.generate_message(_save=False)
+
+    @dualmethod
+    @ModelView.button
+    def generate_message(cls, groups, _save=True):
+        pool = Pool()
+        Message = pool.get('account.payment.sepa.message')
+        for group in groups:
+            tmpl = group.get_sepa_template()
+            if not tmpl:
+                raise NotImplementedError
+            if not group.sepa_messages:
+                group.sepa_messages = ()
+            message = tmpl.generate(group=group,
+                datetime=datetime, normalize=unicodedata.normalize,
+                ).filter(remove_comment).render()
+            message = Message(message=message, type='out', state='waiting',
+                company=group.company)
+            group.sepa_messages += (message,)
+        if _save:
+            cls.save(groups)
 
     @property
     def sepa_initiating_party(self):
@@ -192,8 +208,9 @@ class Payment:
         ondelete='RESTRICT',
         domain=[
             ('party', '=', Eval('party', -1)),
+            ('company', '=', Eval('company', -1)),
             ],
-        depends=['party'])
+        depends=['party', 'company'])
     sepa_mandate_sequence_type = fields.Char('Mandate Sequence Type',
         readonly=True)
     sepa_return_reason_code = fields.Char('Return Reason Code', readonly=True,
@@ -214,15 +231,28 @@ class Payment:
     sepa_instruction_id = fields.Function(fields.Char('SEPA Instruction ID'),
         'get_sepa_instruction_id', searcher='search_sepa_instruction_id')
 
+    @classmethod
+    def copy(cls, payments, default=None):
+        if default is None:
+            default = {}
+        default.setdefault('sepa_mandate_sequence_type', None)
+        return super(Payment, cls).copy(payments, default=default)
+
     @classmethod
     def get_sepa_mandates(cls, payments):
         mandates = []
         for payment in payments:
-            for mandate in payment.party.sepa_mandates:
-                if mandate.is_valid:
-                    break
+            if payment.sepa_mandate:
+                if payment.sepa_mandate.is_valid:
+                    mandate = payment.sepa_mandate
+                else:
+                    mandate = None
             else:
-                mandate = None
+                for mandate in payment.party.sepa_mandates:
+                    if mandate.is_valid:
+                        break
+                else:
+                    mandate = None
             mandates.append(mandate)
         return mandates
 
@@ -260,6 +290,17 @@ class Payment:
                     if number.type == 'iban':
                         return number
 
+    @property
+    def rejected(self):
+        return (self.state == 'failed'
+            and self.sepa_return_reason_code
+            and self.sepa_return_reason_information == '/RTYP/RJCT')
+
+    def create_clearing_move(self, date=None):
+        if not date:
+            date = Transaction().context.get('date_value')
+        return super(Payment, self).create_clearing_move(date=date)
+
 
 class Mandate(Workflow, ModelSQL, ModelView):
     'SEPA Mandate'
@@ -271,6 +312,7 @@ class Mandate(Workflow, ModelSQL, ModelView):
             },
         depends=['state'])
     account_number = fields.Many2One('bank.account.number', 'Account Number',
+        ondelete='RESTRICT',
         states={
             'readonly': Eval('state').in_(['validated', 'canceled']),
             'required': Eval('state') == 'validated',
@@ -443,7 +485,9 @@ class Mandate(Workflow, ModelSQL, ModelView):
     def sequence_type(self):
         if self.type == 'one-off':
             return 'OOFF'
-        elif not self.payments:
+        elif (not self.payments
+                or all(not p.sepa_mandate_sequence_type for p in self.payments)
+                or all(p.rejected for p in self.payments)):
             return 'FRST'
         # TODO manage FNAL
         else:
